// ==============================================================
// ORBITER vessel module: Juno II Pioneer
// ==============================================================

#include "orbitersdk.h"
#include "multi_stage.h"

const double slThrust = 68040*KGF;
const double vacThrust = 77367*KGF;

const VECTOR3 OFS_NOSE = { 0,0,.25};
const VECTOR3 OFS_THIRD = { 0,0,-.87-.35};
const VECTOR3 OFS_SECOND = { 0.0, 0.0, OFS_THIRD.z-1.25};
const VECTOR3 OFS_FIRST = { 0.0, 0.0, OFS_SECOND.z-1-9.5};
bool bManualSeparate = false;
double stage_sep=0.0;
UINT stage = 0;

void SetPioneerStage (VESSEL *vessel)
{
// ==============================================================
	vessel->SetSize (1.5);
	vessel->SetCOG_elev (1.0);
	vessel->SetEmptyMass (6.0);
	vessel->SetMaxFuelMass (21.0);
	vessel->SetFuelMass(21.0);
	vessel->SetISP (235.0*9.8);
	vessel->SetMaxThrust (ENGINE_MAIN, 6700);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5);
	vessel->SetEngineLevel(ENGINE_MAIN,vessel->GetEngineLevel(ENGINE_MAIN));
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(.31,.31,0.01));
	vessel->SetCrossSections (_V(.31,.31,.01));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.3, .3, .3);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
// ==============================================================
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	vessel->AddMesh ("junoII4");
	VECTOR3 m_exhaust_pos= {0,0,-0.85};
	VECTOR3 m_exhaust_ref = {0,0,-1};
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.0, 0.1, &m_exhaust_ref);
}

void SetThirdStage (VESSEL *vessel)
{
// ==============================================================
	vessel->SetSize (1.5);
	vessel->SetCOG_elev (0);
	vessel->SetEmptyMass (56.0);
	vessel->SetMaxFuelMass (21.0*3);
	vessel->SetFuelMass(21.0*3);
	vessel->SetISP (235*9.8);
	vessel->SetMaxThrust (ENGINE_MAIN, 6700.0*3);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e1);
	vessel->SetEngineLevel(ENGINE_MAIN,vessel->GetEngineLevel(ENGINE_MAIN));
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(.4,.4,0.1));
	vessel->SetCrossSections (_V(.41,.41,.91));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.1, 0.1,0.7);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	vessel->AddMesh ("junoII4");
	vessel->AddMesh ("junoII3", &OFS_THIRD);
	VECTOR3 m_exhaust_pos;
	VECTOR3 m_exhaust_ref;
	m_exhaust_pos= _V(0,.15, OFS_THIRD.z-.65);
	m_exhaust_ref = _V(0,0,-1);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.0, 0.1, &m_exhaust_ref);
	m_exhaust_pos= _V(-.15,-.05, OFS_THIRD.z-.65);
	m_exhaust_ref = _V(0,0,-1);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.0, 0.1, &m_exhaust_ref);
	m_exhaust_pos= _V(.15,-.05, OFS_THIRD.z-.65);
	m_exhaust_ref = _V(0,0,-1);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.0, 0.1, &m_exhaust_ref);
}

void SetSecondStage (VESSEL *vessel)
{
	vessel->SetSize (2);
	vessel->SetCOG_elev (0);
	vessel->SetEmptyMass (214.0);
	vessel->SetMaxFuelMass (21.0*11);
	vessel->SetFuelMass(21.0*11);
	vessel->SetISP (235*9.8);
	vessel->SetMaxThrust (ENGINE_MAIN, 6700.0*11);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e1);
	vessel->SetEngineLevel(ENGINE_MAIN,vessel->GetEngineLevel(ENGINE_MAIN));
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(.7,.7,0.34));
	vessel->SetCrossSections (_V(2.41,2.41,.91));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.1, 0.1,0.7);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	vessel->AddMesh ("junoII4");
	vessel->AddMesh ("junoII3", &OFS_THIRD);
	vessel->AddMesh ("junoII2", &OFS_SECOND);
	VECTOR3 m_exhaust_pos= {0,0,OFS_SECOND.z-1.35};
	VECTOR3 m_exhaust_ref = {0,0,-1};
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 2.0, .5, &m_exhaust_ref);
}

void SetSecondNoseConeStage (VESSEL *vessel)
{
	vessel->SetSize (2);
	vessel->SetCOG_elev (0);
	vessel->SetEmptyMass (222.0+234.0);
	vessel->SetMaxFuelMass (21.0*11);
	vessel->SetFuelMass(21.0*11);
	vessel->SetISP (235*SEC);
	vessel->SetMaxThrust (ENGINE_MAIN, 6700.0*11);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e1);
	vessel->SetEngineLevel(ENGINE_MAIN,vessel->GetEngineLevel(ENGINE_MAIN));
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(.7,.7,0.34));
	vessel->SetCrossSections (_V(2.41,2.41,.91));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.1, 0.1,0.7);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	VECTOR3 mesh_loc = {0,0,0};
	vessel->AddMesh ("junoII4");
	vessel->AddMesh ("junoII3", &OFS_THIRD);
	vessel->AddMesh ("junoII2", &OFS_SECOND);
	vessel->AddMesh ("junoII2b", &OFS_NOSE);
	VECTOR3 m_exhaust_pos= {0,0,-1.05-.66-1.66};
	VECTOR3 m_exhaust_ref = {0,0,-1};
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.0, 0.1, &m_exhaust_ref);
}


void SetFirstStage (VESSEL *vessel)
{
// ==============================================================
	vessel->SetSize (10);
	vessel->SetCOG_elev (9.25+.83+.33+1);
	vessel->SetEmptyMass (5443.0);
	vessel->SetMaxFuelMass (48988);
	vessel->SetFuelMass(48988);
	vessel->SetISP (282.0*SEC);
	vessel->SetMaxThrust (ENGINE_MAIN, vacThrust);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e3);
	vessel->SetEngineLevel(ENGINE_MAIN,vessel->GetEngineLevel(ENGINE_MAIN));
// ==============================================================
// This is from the altanis launch config and is what makes the V-2 launch vertically
// The Z values you use to set the rocket up into the air and align with the height of 
// your launch platform. In this case the Pad 39A.
	vessel->SetTouchdownPoints (_V(0,-1.0,-48.0), _V(-.7,.7,-48.0), _V(.7,.7,-48.0));
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(25.3,25.3,0.54));
	vessel->SetCrossSections (_V(30.4,30.4,1.74));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.3, 1.4,1.4);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
// ==============================================================
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	vessel->AddMesh ("junoII4");
	vessel->AddMesh ("junoII3", &OFS_THIRD);
	vessel->AddMesh ("junoII2", &OFS_SECOND);
	vessel->AddMesh ("junoII2b", &OFS_NOSE);
	vessel->AddMesh ("junoII1", &OFS_FIRST);
	VECTOR3 m_exhaust_pos;
	VECTOR3 m_exhaust_ref;
	m_exhaust_pos= _V(0,0, OFS_FIRST.z-9.5);
	m_exhaust_ref = _V(0,0,-1);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 4.0, 0.5, &m_exhaust_ref);
}

void SeparateStage (VESSEL *vessel, UINT stage)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,0);

	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;

	if (stage == 0)
	{
		ofs = OFS_FIRST;	// The same as whatever offset you used to place the stage 
		vel = _V(0,0,-1);	// Staging Velocity represents 
		StageTransform(vessel,&vs,ofs,vel);
		vs.vrot.x = 0.0;	//<- Use these to rotate the ejected stage
		vs.vrot.y = 0.0;	//<-
		vs.vrot.z = 0.0;	//<-
		char name[256];
		strcpy (name, vessel->GetName()); strcat (name, "-FirstStage");
		VESSEL::Create (name, "JunoIIFirstStg", vs);
		SetSecondNoseConeStage(vessel);
		vessel->SetEngineLevel(ENGINE_MAIN, 0);

	}
	else if (stage == 1)
	{
		ofs = OFS_NOSE;	// The same as whatever offset you used to place the stage 
		vel = _V(0,0,1);	// Staging Velocity represents 
		StageTransform(vessel,&vs,ofs,vel);
		vs.vrot.x = 0.0;	//<- Use these to rotate the ejected stage
		vs.vrot.y = 0.0;	//<-
		vs.vrot.z = 0.0;	//<-
		char name[256];
		strcpy (name, vessel->GetName()); strcat (name, "-NoseCone");
		VESSEL::Create (name, "JunoIINoseCone", vs);
		SetSecondStage (vessel);
		vessel->SetEngineLevel(ENGINE_MAIN, 0);
	}
	else if (stage == 2)
	{
		ofs = OFS_SECOND;	// The same as whatever offset you used to place the stage 
		vel = _V(0,0,-1);	// Staging Velocity represents 
		StageTransform(vessel,&vs,ofs,vel);
		vs.vrot.x = 0.0;	//<- Use these to rotate the ejected stage
		vs.vrot.y = 0.0;	//<-
		vs.vrot.z = 0.0;	//<-
		char name[256];
		strcpy (name, vessel->GetName()); strcat (name, "-SecondStage");
		VESSEL::Create (name, "JunoIISecondStg", vs);
		SetThirdStage (vessel);
	}
	else if (stage == 3)
	{
		ofs = OFS_THIRD;	// The same as whatever offset you used to place the stage 
		vel = _V(0,0,-1);	// Staging Velocity represents 
		StageTransform(vessel,&vs,ofs,vel);
		vs.vrot.x = 0.0;	//<- Use these to rotate the ejected stage
		vs.vrot.y = 0.0;	//<-
		vs.vrot.z = 0.0;	//<-
		char name[256];
		strcpy (name, vessel->GetName()); strcat (name, "-ThirdStage");
		VESSEL::Create (name, "JunoIIThirdStg", vs);
		SetPioneerStage (vessel);
	}

}







// ==============================================================
// API interface
// ==============================================================



// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	stage = 0;
	return new VESSEL (hvessel, flightmodel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete vessel;
}

// Keyboard interface handler
DLLCLBK int ovcConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bManualSeparate = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_DIVIDE)) { // "Disable Laterals"
			oapiSetAttitudeMode(vessel->GetHandle(),0);
			return 1;
		}
	}
	return 0;
}

// Set the capabilities of the vessel class
DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	SetFirstStage(vessel);
}


DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
{
	if (stage == 0)
	{
		DoThrustAdjust(vessel,slThrust,vacThrust);
		if (vessel->GetFuelMass() == 0 || bManualSeparate)
		{
			stage_sep = simt;
			SeparateStage (vessel, stage);
			bManualSeparate=false;
			stage = 1;
		} 
	}
	else if (stage == 1)
	{
		if (vessel->GetAtmPressure() < .01 || bManualSeparate)
		{
			stage_sep = simt;
			bManualSeparate=false;
			SeparateStage (vessel, stage);
			stage = 2;
		}
	}
	else if (stage == 2)
	{
		if (vessel->GetFuelMass() == 0 || bManualSeparate)
		{
			stage_sep = simt;
			bManualSeparate=false;
			SeparateStage (vessel, stage);
			stage = 3;
		}
	}
	else if (stage == 3)
	{
		if (vessel->GetFuelMass() == 0 || bManualSeparate)
		{
			stage_sep = simt;
			bManualSeparate=false;
			SeparateStage (vessel, stage);
			stage = 4;
		}
	}
	else if (stage == 4)
	{
	}


}